import React, { useState, useCallback, useEffect } from 'react';
import ReactDOM from 'react-dom';
import { csv, arc, pie, scaleBand, scaleLinear, max, format } from 'd3';
import { useData } from './useData';
import { AxisBottom } from './AxisBottom';
import { AxisLeft } from './AxisLeft';
import { Marks } from './Marks';

const width = 960;
const height = 380;
const margin = { top: 20, right: 30, bottom: 65, left: 180 }; // making room for axis and gaps through margin
const xAxisLabelOffset = 50;

const App = () => {
  const data = useData();

  if (!data) {
    return <pre>Loading...</pre>;
  }

  const innerHeight = height - margin.top - margin.bottom; // creating gap on bottom
  const innerWidth = width - margin.left - margin.right; // creating gap on right

  const yValue = d => d.Company;
  const xValue = d => d.CompanyValue;

  const siFormat = format('.4s');
  const xAxisTickFormat = tickValue => siFormat(tickValue).replace('G', 'B');
// invoking scaleband constructor
  const yScale = scaleBand()
    .domain(data.map(yValue))// data as input and returns company name
    .range([0, innerHeight])// setting the range as height from top to bottom
    .paddingInner(0.15);
// invoking scalelinear constructor
  const xScale = scaleLinear()
    .domain([0, max(data, xValue)]) //setting domain from 0 to maximun revenue
    .range([0, innerWidth]);

  return (
    // moving all elements in the group using translate and transform functions
    <svg width={width} height={height}>
      <g transform={`translate(${margin.left},${margin.top})`}>
        <AxisBottom
          xScale={xScale}
          innerHeight={innerHeight}
          tickFormat={xAxisTickFormat}
        />
        <AxisLeft yScale={yScale} />
        <text
          className="axis-label"
          x={innerWidth / 2}
          y={innerHeight + xAxisLabelOffset}
          textAnchor="middle"
        >
          Company Value
        </text>
        <Marks
          data={data}
          xScale={xScale}
          yScale={yScale}
          xValue={xValue}
          yValue={yValue}
          tooltipFormat={xAxisTickFormat}
        />
      </g>
    </svg>
  );
};
const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
